#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Module is containing all necessary global variables for the package.

Module also has the ability to read user-defined data from two paths:

- ``$HOME/_SETTINGS_PATH``
- ``/etc/_SETTINGS_PATH``

See :attr:`._SETTINGS_PATH` for details.

Note:
    If the first path is found, other is ignored.

Example of the configuration file (``$HOME/edeposit/urnnbn.json``)::

    {
        "EXPORT_DIR": "/somedir/somewhere"
    }

Attributes
----------
"""
import json
import os
import os.path
from urlparse import urljoin


# Module configuration ========================================================


#: Username for the URN:NBN resolver website
USERNAME = ""

#: Password for the URN:NBN resolver website
PASSWORD = ""

#: Registration code (used in API)
REG_CODE = "edep"

#: URL of the URN:NBN resolver
URL = "https://resolver-test.nkp.cz/api/v3/"

#: 
REG_URL = urljoin(URL, "registrars/")


# User configuration reader (don't edit this ==================================
_ALLOWED = [unicode, str, int, float]

_SETTINGS_PATH = "/edeposit/urnnbn.json"
"""
Path which is appended to default search paths (``$HOME`` and ``/etc``).

Note:
    It has to start with ``/``. Variable is **appended** to the default search
    paths, so this doesn't mean, that the path is absolute!
"""


def get_all_constants():
    """
    Get list of all uppercase, non-private globals (doesn't start with ``_``).

    Returns:
        list: Uppercase names defined in `globals()` (variables from this \
              module).
    """
    return filter(
        lambda key: key.upper() == key and type(globals()[key]) in _ALLOWED,

        filter(                               # filter _PRIVATE variables
            lambda x: not x.startswith("_"),
            globals()
        )
    )


def substitute_globals(config_dict):
    """
    Set global variables to values defined in `config_dict`.

    Args:
        config_dict (dict): dictionary with data, which are used to set \
                            `globals`.

    Note:
        `config_dict` have to be dictionary, or it is ignored. Also all
        variables, that are not already in globals, or are not types defined in
        :attr:`_ALLOWED` (str, int, float) or starts with ``_`` are silently
        ignored.
    """
    constants = get_all_constants()

    if type(config_dict) != dict:
        return

    for key in config_dict:
        if key in constants and type(config_dict[key]) in _ALLOWED:
            globals()[key] = config_dict[key]


# try to read data from configuration paths ($HOME/_SETTINGS_PATH,
# /etc/_SETTINGS_PATH)
if "HOME" in os.environ and os.path.exists(os.environ["HOME"] + _SETTINGS_PATH):
    with open(os.environ["HOME"] + _SETTINGS_PATH) as f:
        substitute_globals(json.loads(f.read()))
elif os.path.exists("/etc" + _SETTINGS_PATH):
    with open("/etc" + _SETTINGS_PATH) as f:
        substitute_globals(json.loads(f.read()))



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from modes import Modes
from catalog import Catalog
from digital_library import DigitalLibrary

from tools import to_list
from tools import both_set_and_different


# Functions & classes =========================================================
class Registrar(object):
    """
    Class holding informations about Registrar.

    Attributes:
        uid (str): Id of the registrar in URN:NBN system.
        code (str): Code of the registrar. Each organization has own.
        name (str): Full name of the registrar.
        created (str): ISO 8601 date string.
        modified (str): ISO 8601 date string.
        description (str): Description of the registrar.
        modes (obj): :class:`.Modes` instance with informations about allowed
                     modes.
        catalogs (list): List of :class:`.Catalog` instances with informations
                 about catalogs used by this registrar.
        digital_libraries (list): List of :class:`.DigitalLibrary` instances
                          with informations about digital libraries used by
                          registrar.
    """
    def __init__(self, code, uid, name=None, description=None, created=None,
                 modified=None, modes=None):
        self.uid = uid
        self.code = code
        self.name = name
        self.created = created
        self.modified = modified
        self.description = description

        self.modes = modes
        self.catalogs = []
        self.digital_libraries = []

    def __repr__(self):
        return "%s:%s" % (self.__class__.__name__, self.code)

    def __eq__(self, other):
        if not (self.uid == other.uid and self.code == other.code):
            return False

        not_important_checks = any([
            both_set_and_different(self.name, other.name),
            both_set_and_different(self.description, other.description),
            both_set_and_different(self.created, other.created),
            both_set_and_different(self.modified, other.modified),
            both_set_and_different(self.modes, other.modes),
        ])
        if not_important_checks:
            return False

        return all([
            set(self.catalogs) == set(other.catalogs),
            set(self.digital_libraries) == set(other.digital_libraries)
        ])

    def __ne__(self, other):
        return not self.__eq__(other)

    @staticmethod
    def from_xmldict(reg_tag):
        """
        Parse basic information about registrar.

        Args:
            reg_tag (obj): OrderedDict returned from :mod:`xmltodict`.

        Returns:
            obj: :class:`.Registrar` instance with basic informations.
        """
        # parse Registrar data
        registrar = Registrar(
            code=reg_tag["@code"],
            uid=reg_tag["@id"],
            name=reg_tag["name"],
            description=reg_tag.get("description", None),
            created=reg_tag["created"],
            modified=reg_tag.get("modified", None),
            modes=Modes.from_xmldict(
                reg_tag["registrationModes"]["mode"]
            )
        )

        if not reg_tag.get("digitalLibraries", None):
            return registrar

        # parse digital_libraries
        for dl_tag in to_list(reg_tag["digitalLibraries"]["digitalLibrary"]):
            registrar.digital_libraries.append(
                DigitalLibrary(
                    uid=dl_tag["@id"],
                    name=dl_tag["name"],
                    description=dl_tag.get("description", None),
                    url=dl_tag.get("url", None),
                    created=dl_tag.get("created", None),
                )
            )

        if not reg_tag.get("catalogs", None):
            return registrar

        # parse catalogs
        for catalog_tag in to_list(reg_tag["catalogs"]["catalog"]):
            registrar.catalogs.append(
                Catalog(
                    uid=catalog_tag["@id"],
                    name=catalog_tag["name"],
                    url_prefix=catalog_tag.get("urlPrefix", None),
                    created=catalog_tag.get("created", None),
                )
            )

        return registrar



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
"""
This module contains convertors for converting MODS to XML required by URN\:NBN
project.

See:
    - http://resolver.nkp.cz/api/v3/digDocRegistration.xsd
    - https://code.google.com/p/czidlo/wiki/ApiV3
"""
# Imports =====================================================================
import xmltodict
import dhtmlparser
from dhtmlparser import first

from xml_composer import MonographComposer
from xml_composer import MultiMonoComposer


# Functions & classes =========================================================
def pick_only_text(fn):
    def get_only_text(*args, **kwargs):
        out = fn(*args, **kwargs)

        if not out:
            return out

        if "#text" in out:
            return out["#text"]

        return out

    return get_only_text


def first_or_none(fn):
    def get_first_or_none(*args, **kwargs):
        out = fn(*args, **kwargs)

        if not out:
            return None

        return first(out).getContent().decode("utf-8")

    return get_first_or_none


class MonographPublication(object):
    """
    This class accepts MODS monographic data, which can then convert to XML
    for URN\:NBN.
    """
    def __init__(self, mods_xml):
        self.mods_xml = mods_xml
        self.dom = dhtmlparser.parseString(mods_xml)
        self.xdom = xmltodict.parse(mods_xml)

        self.composer = MonographComposer()

    def _get_title_info(self):
        return self.dom.wfind(
            "mods:mods"
        ).wfind(
            "mods:titleInfo",
            fn=lambda x: x.params.get("type") != "alternative"
        )

    @first_or_none
    def get_title(self):
        """
        Returns:
            str: Title
        """
        titles = self._get_title_info().find("mods:title")

        if not titles:
            ValueError("<mods:title> element not found.")

        return titles

    @first_or_none
    def get_subtitle(self):
        """
        Returns:
            str: Subtitle
        """
        return self._get_title_info().find("mods:subTitle")

    @first_or_none
    def get_author(self):
        """
        Returns:
            str: Author's name.
        """
        author = self.dom.match(
            "mods:mods",
            ["mods:name", {"type": "personal", "usage": "primary"}],
            {
                "tag_name": "mods:namePart",
                "fn": lambda x: x.params.get("type", "") != "date"
            }
        )
        if not author:
            author = self.dom.match(
                "mods:mods",
                ["mods:name", {"type": "corporate"}]
            )
        if not author:
            author = self.dom.match(
                "mods:mods",
                ["mods:name", {"type": 'conference'}]
            )

        return author

    def get_form(self):
        """
        Returns:
            str: Form of the book. Electronic source, and so on..
        """
        forms = self.dom.match(
            "mods:mods",
            "mods:physicalDescription",
            {
                "tag_name": "mods:form",
                "fn": lambda x: x.params.get("authority", "") == "gmd"
            }
        )
        if not forms:
            return

        return forms[0].getContent().decode("utf-8")

    def _get_description(self):
        return self.xdom["mods:mods"].get("mods:originInfo", None)

    def get_place(self):
        """
        Returns:
            str: Place where the book was released.
        """
        place = self.dom.match(
            "mods:originInfo",
            "mods:place",
            ["mods:placeTerm", {"type": "text"}]
        )
        if not place:
            return

        return place[0].getContent().decode("utf-8")

    @pick_only_text
    def get_publisher(self):
        """
        Returns:
            str: Name of the publisher.
        """
        if not self._get_description():
            return

        return self._get_description().get("mods:publisher", None)

    @pick_only_text
    def get_year(self):
        """
        Returns:
            str: Year when the book was released.
        """
        if not self._get_description():
            return

        return self._get_description().get("mods:dateIssued", None)

    @pick_only_text
    def get_identifier(self, name):
        """
        Returns:
            str: Identifier from ``<mods:identifier>`` which has \
                 ``@type == name``.
        """
        identifier = filter(
            lambda x: x.get("@type", False) == name,
            self.xdom["mods:mods"].get("mods:identifier", [])
        )

        if not identifier:
            return

        return identifier[0]

    @pick_only_text
    def get_ccnb(self):
        """
        Returns:
            str: CCNB identification string.
        """
        return self.get_identifier("ccnb")

    @pick_only_text
    def get_isbn(self):
        """
        Returns:
            str: ISBN.
        """
        return self.get_identifier("isbn")

    @pick_only_text
    def get_uuid(self):
        """
        Returns:
            str: UUID.
        """
        return self.get_identifier("uuid")

    def compose(self):
        """
        Convert `self` to nested ordered dicts, which may be serialized to XML
        using ``xmltodict`` module.

        Returns:
            OrderedDict: XML parsed to ordered dicts.
        """
        self.composer.title = self.get_title()
        self.composer.subtitle = self.get_subtitle()
        self.composer.ccnb = self.get_ccnb()
        self.composer.isbn = self.get_isbn()
        self.composer.other_id = self.get_uuid()
        self.composer.document_type = self.get_form()
        self.composer.digital_born = True
        self.composer.author = self.get_author()
        self.composer.place = self.get_place()
        self.composer.publisher = self.get_publisher()
        self.composer.year = self.get_year()

    def add_format(self, file_format):
        """
        Add informations about `file_format` to internal XML dict.

        Args:
            file_format (str): ``PDF``, ``jpeg``, etc..
        """
        self.composer.format = file_format

    def to_xml(self):
        """
        Convert itself to XML unicode string.

        Returns:
            unicode: XML.
        """
        self.compose()
        return self.composer.to_xml()

    def __str__(self):
        return self.to_xml()


class MonographVolume(MonographPublication):
    """
    Conversion of Multi-monograph data to XML required by URN\:NBN.
    """
    def __init__(self, mods_xml):
        super(MonographVolume, self).__init__(mods_xml)

        self.composer = MultiMonoComposer()

    @pick_only_text
    def get_volume_title(self):
        """
        Returns:
            str: Title of the whole volume.
        """
        title_info = self.dom.match(
            "mods:mods",
            "mods:titleInfo",
            "mods:partNumber"
        )

        if not title_info:
            title_info = self.dom.match(
                "mods:mods",
                "mods:titleInfo",
                "mods:partName"
            )

        if not title_info:
            raise ValueError("Can't find volumeTitle!")

        return title_info[0].getContent().decode("utf-8")

    def compose(self):
        """
        Convert `self` to nested ordered dicts, which may be serialized to XML
        using ``xmltodict`` module.

        Returns:
            OrderedDict: XML parsed to ordered dicts.
        """
        super(MonographVolume, self).compose()
        self.composer.volume_title = self.get_volume_title()


def convert_mono_xml(mods_xml, file_format):
    """
    Convert MODS monograph record to XML, which is required by URN:NBN
    resolver.

    Args:
        mods_xml (str): MODS volume XML.

    Returns:
        str: XML for URN:NBN resolver.

    Raises:
        ValueError: If can't find required data in MODS (author, title).
    """
    pub = MonographPublication(mods_xml)
    pub.add_format(file_format)

    return pub.to_xml()


def convert_mono_volume_xml(mods_volume_xml, file_format):
    """
    Convert MODS monograph, multi-volume record to XML, which is required by
    URN:NBN resolver.

    Args:
        mods_volume_xml (str): MODS volume XML.

    Returns:
        str: XML for URN:NBN resolver.

    Raises:
        ValueError: If can't find required data in MODS (author, title).
    """
    pub = MonographVolume(mods_volume_xml)
    pub.add_format(file_format)

    return pub.to_xml()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================


# Functions & classes =========================================================
def both_set_and_different(first, second):
    """
    If any of both arguments are unset (=``None``), return ``False``. Otherwise
    return result of unequality comparsion.

    Returns:
        bool: True if both arguments are set and different.
    """
    if first is None:
        return False

    if second is None:
        return False

    return first != second


def to_list(tag):
    """
    Put `tag` to list if it ain't list/tuple already.

    Args:
        tag (obj): Anything.

    Returns:
        list: Tag.
    """
    if isinstance(tag, tuple) or isinstance(tag, list):
        return tag

    return [tag]



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from __future__ import unicode_literals

import pytest

from cz_urnnbn_api import api
from cz_urnnbn_api import api_structures

from test_xml_composer import data_context
from test_xml_composer import mono_out_example


# Fixtures ====================================================================
@pytest.fixture
def iter_registrars_data():
    return data_context("iter_registrars_data.xml")


@pytest.fixture
def registrar_data():
    return data_context("registrar_data.xml")


@pytest.fixture
def registration_response():
    return data_context("registration_response.xml")


# Tests =======================================================================
def test_is_valid_reg_code():
    assert api.is_valid_reg_code()
    assert not api.is_valid_reg_code("azgabash")


def test_iter_registrars(monkeypatch, iter_registrars_data):
    def send_request(*args, **kwargs):
        return iter_registrars_data

    monkeypatch.setattr(api, "_send_request", send_request)

    registrars = list(api.iter_registrars())

    assert len(registrars) == 2

    assert registrars[0] == api_structures.Registrar(
        code="boa001",
        uid="8",
        name="MZK",
        description="Moravská zemská knihovna",
        created="2012-04-18T23:19:17.727+02:00",
        modified="2012-10-29T10:39:30.882+01:00",
        modes=api_structures.Modes(
            by_resolver=True,
            by_registrar=True,
            by_reservation=True,
        )
    )

    assert registrars[1] == api_structures.Registrar(
        code="aba001",
        uid="9",
        name="NKP",
        description="Národní knihovna Praha",
        created="2012-04-18T23:19:16.143+02:00",
        modes=api_structures.Modes(
            by_resolver=False,
            by_registrar=True,
            by_reservation=False,
        )
    )

    assert registrars[1] != api_structures.Registrar(
        code="aba001",
        uid="9",
        name="NKP",
        description="Národní knihovna Praha",
        created="2012-04-18T23:19:16.143+02:00",
        modes=api_structures.Modes(
            by_resolver=True,
            by_registrar=True,
            by_reservation=False,
        )
    )

    assert registrars[1] != api_structures.Registrar(
        code="aba001a",
        uid="9",
        name="NKP",
        description="Národní knihovna Praha",
        created="2012-04-18T23:19:16.143+02:00",
        modes=api_structures.Modes(
            by_resolver=False,
            by_registrar=True,
            by_reservation=False,
        )
    )


def test_get_registrar_info(monkeypatch, registrar_data):
    def send_request(*args, **kwargs):
        return registrar_data

    monkeypatch.setattr(api, "_send_request", send_request)

    registrar = api.get_registrar_info("asd")

    assert registrar.name == "Moravská zemská knihovna - stará digitalizace"
    assert registrar.description == "Před rokem 2012."
    assert registrar.created == "2012-04-11T11:15:37.613+02:00"
    assert registrar.modified == "2013-01-10T09:01:03.690+01:00"
    assert registrar.name == "Moravská zemská knihovna - stará digitalizace"
    assert registrar.name == "Moravská zemská knihovna - stará digitalizace"

    assert registrar.modes == api_structures.Modes(
        by_resolver=True,
        by_registrar=True,
        by_reservation=False,
    )

    assert len(registrar.digital_libraries) == 2

    assert registrar.digital_libraries[0] == api_structures.DigitalLibrary(
        uid="13",
        name="Kramerius 4",
        description="aktuální verze knihovny Kramerius",
        url="http://kramerius.mzk.cz",
        created="2012-04-11T11:15:37.638+02:00",
    )

    assert registrar.digital_libraries[1] == api_structures.DigitalLibrary(
        uid="14",
        name="Kramerius 3",
        description="předchozí verze knihovny Kramerius",
        url="http://kramerius3.mzk.cz",
        created="2012-04-11T11:15:37.660+02:00",
    )

    assert len(registrar.catalogs) == 1

    assert registrar.catalogs[0] == api_structures.Catalog(
        uid="2",
        name="Aleph MZK",
        created="2012-04-11T11:15:38.525+02:00",
        url_prefix="http://aleph.mzk.cz"
    )


def test_register_document(monkeypatch, registration_response):
    def send_request(*args, **kwargs):
        return registration_response

    monkeypatch.setattr(api, "_send_request", send_request)

    response = api.register_document("something")

    assert response == "urn:nbn:cz:edep-00000j"

    assert response.value == "urn:nbn:cz:edep-00000j"
    assert response.status == "ACTIVE"
    assert response.registered == "2015-04-29T17:43:48.399+02:00"
    assert response.country_code == "cz"
    assert response.document_code == "00000j"
    assert response.registrar_code == "edep"
    assert response.digital_document_id == "52974"



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from api_structures import *

from xml_composer import MonographComposer
from xml_composer import MultiMonoComposer

from xml_convertor import convert_mono_xml
from xml_convertor import convert_mono_volume_xml




#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
import os
import os.path

import pytest

from cz_urnnbn_api.xml_convertor import convert_mono_xml
from cz_urnnbn_api.xml_convertor import MonographPublication
from cz_urnnbn_api.xml_convertor import convert_mono_volume_xml

from test_xml_composer import data_context
from test_xml_composer import get_data_path
from test_xml_composer import mono_out_example
from test_xml_composer import multimono_out_example


# Fixtures ====================================================================
@pytest.fixture
def mono_mods_example():
    return data_context("mods_mono.xml")


# Tests =======================================================================
def test_mono_xml_conversion(mono_mods_example, mono_out_example):
    out = convert_mono_xml(mono_mods_example, "pdf")

    assert out == mono_out_example


def test_mono_volume_xml_conversion(mono_mods_example, multimono_out_example):
    out = convert_mono_volume_xml(mono_mods_example, "pdf")

    assert out == multimono_out_example


def test_year_with_attribute():
    example = """<?xml version="1.0" encoding="UTF-8"?>
<mods:mods>
  <mods:originInfo>
    <mods:place>
      <mods:placeTerm type="code" authority="marccountry">xr-</mods:placeTerm>
    </mods:place>
    <mods:dateIssued encoding="marc">2011</mods:dateIssued>
    <mods:edition>1. elektronické vydání</mods:edition>
    <mods:issuance>multipart monograph</mods:issuance>
  </mods:originInfo>
</mods:mods>"""

    mp = MonographPublication(example)

    assert mp.get_year() == "2011"


def test_additional_conversions():
    example_path = os.path.join(get_data_path(), "additional_examples")

    example_paths = (
        os.path.join(example_path, fn)
        for fn in os.listdir(example_path)
    )

    examples = (
        open(fn).read()
        for fn in example_paths
        if os.path.isfile(fn)
    )

    for example in examples:
        assert convert_mono_xml(example, "pdf")



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
import pytest

from cz_urnnbn_api.api_structures import tools


# Tests =======================================================================
def test_to_list():
    assert tools.to_list(1) == [1]

    assert tools.to_list([1]) == [1]
    assert tools.to_list((1,)) == (1,)


def test_both_set_and_different():
    assert not tools.both_set_and_different(None, 1)
    assert not tools.both_set_and_different(1, None)
    assert not tools.both_set_and_different(None, None)

    assert not tools.both_set_and_different(1, 1)
    assert tools.both_set_and_different(1, 2)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================


# Functions & classes =========================================================
def _by_attr(xdom, attr):
    """
    From `xdom` pick element with attributes defined by `attr`.

    Args:
        xdom (obj): DOM parsed by :mod:`xmltodict`.
        attr (dict): Dictionary defining all the arguments.

    Returns:
        obj: List in case that multiple records were returned, or OrderedDict \
             instance in case that there was only one. Blank array in case of \
             no matching tag.
    """
    out = []

    for tag in xdom:
        for attr_name, val in attr.iteritems():
            if attr_name not in tag:
                break

            if val is not None and tag[attr_name] != val:
                break

            out.append(tag)

    return out[0] if len(out) == 1 else out


class Modes(object):
    """
    Container holding informations about modes which may be used by registrar
    to register documents.

    Attributes:
        by_resolver (bool): True if the mode can be used.
        by_registrar (bool): True if the mode can be used.
        by_reservation (bool): True if the mode can be used.
    """
    def __init__(self, by_resolver=False, by_registrar=False,
                 by_reservation=False):
        self.by_resolver = by_resolver
        self.by_registrar = by_registrar
        self.by_reservation = by_reservation

    def __eq__(self, other):
        return all([
            self.by_resolver == other.by_resolver,
            self.by_registrar == other.by_registrar,
            self.by_reservation == other.by_reservation,
        ])

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return "Modes(%r%r%r)" % (
            self.by_resolver,
            self.by_registrar,
            self.by_reservation
        )

    @staticmethod
    def from_xmldict(modes_tag):
        """
        Parse Modes information from XML.

        Args:
            modes_tags (obj): OrderedDict ``<modes>`` tag returned from
                      :mod:`xmltodict`.

        Returns:
            obj: :class:`.Modes` instance.
        """
        by_resolver = _by_attr(modes_tag, attr={"@name": "BY_RESOLVER"})
        by_registrar = _by_attr(modes_tag, attr={"@name": "BY_REGISTRAR"})
        by_reservation = _by_attr(modes_tag, attr={"@name": "BY_RESERVATION"})

        return Modes(
            by_resolver=by_resolver["@enabled"].lower() == "true",
            by_registrar=by_registrar["@enabled"].lower() == "true",
            by_reservation=by_reservation["@enabled"].lower() == "true",
        )
